myname = 'compare_orig_vs_ds_change.R'

# leave false for 1-16 deg vers
do_BC_version = FALSE

source( '~/location.R' )
source( '../mod_descrip_v10.R' )
source( '../make_all_filenames_v4.R' )

source('~/src/mine/R/multi_page_png.R' )

seas = c('DJF', 'MAM', 'JJA', 'SON')

period_pat       = c('2006-2039', '2040-2069', '2070-2???' )
period_pat_label = c('2006-2039', '2040-2069', '2070-2100' )

test_no_plots = FALSE

#-------------------------------------------------------------------
# Since we deal with original model grids here, I make a common grid 
# and regrid all the orig model grids to it
#-------------------------------------------------------------------
xvals_com = seq( -125.5, -66.5, by=1 )
yvals_com = seq(   22.5,  53.5, by=1 )

#======================================================================================
get_delta = function( fname_fut, fname_hist, modd, ivar, regrid=FALSE ) {

	print(paste('fname hist:', fname_hist ))
	print(paste('fname fut:', fname_fut ))

	if( test_no_plots ) {
		nxc = length( xvals_com )
		nyc = length( yvals_com )
		delta = rnorm( nxc*nyc )
		dim( delta ) = c( nxc, nyc )
		return( list(
			delta		= delta,
			xvals		= xvals_com,
			yvals		= yvals_com ))
		}

	ncid_hist = nc_open( fname_hist )
	var_hist  = ncid_hist$var[[1]]
	dat_hist  = ncvar_get( ncid_hist, var_hist  )
	ndims_hist = length(dim(dat_hist))

	ncid_fut  = nc_open( fname_fut )
	dat_fut   = ncvar_get( ncid_fut )
	ndims_fut = length(dim(dat_fut))

	if( (ndims_hist != 2) || (ndims_fut != 2))
		stop(paste("Error, ndims hist and fut must be 2"))

	xvals = var_hist$dim[[1]]$vals
	yvals = var_hist$dim[[2]]$vals

	if( max(xvals) > 181 )
		xvals = xvals - 360

	if( ivar == IVARTYPE_PR ) 
		delta = (dat_fut - dat_hist)/dat_hist * 100.
	else	
		delta = dat_fut - dat_hist

	nc_close( ncid_hist )
	nc_close( ncid_fut  )

	#------------------------------
	# Implement regrid if requested
	#------------------------------
	if( regrid ) {
		mask_isgood_wanted = array( TRUE, dim=c(length(xvals_com), length(yvals_com)))
		delta = put_on_common_grid( xvals_com, yvals_com, mask_isgood_wanted, xvals, yvals, delta )
		xvals = xvals_com
		yvals = yvals_com
		}

	return( list(
		delta		= delta,
		xvals		= xvals,
		yvals		= yvals ))
}

#======================================================================================
add_mean = function( ff ) {

	ffm = mean( ff, na.rm=TRUE )

	rmse = sqrt( mean( ff**2, na.rm=TRUE ))

	s = paste( 'Mean:', mformatC('%.1f', ffm ), '   RMSE:', mformatC('%.1f', rmse) )

	text( -124, 51, s, adj=c(0,0), cex=1.8)
}

#======================================================================================
domodel_var = function( fname_all, modd, imod, ivar, is_last_mod=is_last_mod, ret_all=ret_all ) {

	nseas = length( seas )

	vv = modd$var[[ivar]]
	if( ! is_last_mod ) 
		mm = modd$model[[imod]]

	basedir = '/nobackupp6/nexprojects/LOCAds2014'
	datadir_ds  = paste0( basedir, '/downscale_v14' )
	livdir  = paste0( basedir, '/Livneh/livneh_NAmerExt_15Oct2014' )

	#---------------------------------------
	# Compare historical monthly mean to obs
	#---------------------------------------
	fname_clim_liv = paste0( livdir, '/', 
		vv$varname_livneh, '.1950-2005.', vv$units_tag, '.crop.preproc_obs.', 
		vv$varname_livneh, '_only.monthly.clim.nc' )

	idstype = IDSTYPE_16DEG
	iexpt = IEXPT_HISTORICAL
	iperiod = IPERIOD_TRAINING_PERIOD

	#------------------------------------------
	# Get model-predicted mean change by season
	#------------------------------------------
	idstype = IDSTYPE_16DEG
	nperiod = length( period_pat )

	nexpt = length( modd$expt )

	#---------------------------------------------------------------
	# Our returned plots ... [[expt]][[period]][[season]][[3]]$x|y|z
	# where the final '3' is 1 for orig model, '2' for BC model,
	# and '3' for error
	#---------------------------------------------------------------
	return_plots = list()
	for( iexpt in 2:nexpt )
		return_plots[[iexpt]] = list()

	for( iexpt in 2:nexpt )
	for( iperiod in 1:nperiod ) {

		return_plots[[iexpt]][[iperiod]] = list()

		if( ! is_last_mod ) {
			#----------------------
			# AFTER bias correction
			#----------------------
			fname_base = fname_all$dstype[[idstype]]$mod[[imod]]$var[[ivar]]$expt[[iexpt]]$period[[iperiod]]$ds
			bn = basename( fname_base, '.nc' )
			dn = dirname ( fname_base )

			fname_base_hist = fname_all$dstype[[idstype]]$mod[[imod]]$var[[ivar]]$expt[[IEXPT_HISTORICAL]]$period[[IPERIOD_TRAINING_PERIOD]]$ds
			bnh = basename( fname_base_hist, '.nc' )
			dnh = dirname ( fname_base_hist )

			bnh = paste0( bnh, '.postdsbc' )
			bn  = paste0( bn , '.postdsbc' )

			#-----------------------
			# BEFORE bias correction
			#-----------------------
			fname_base_orig = paste0(datadir_ds,'/',fname_all$dstype[[IDSTYPE_1x1]]$mod[[imod]]$var[[ivar]]$expt[[iexpt]]$period[[iperiod]]$stdcal)
			fname_base_hist_orig = paste0(datadir_ds,'/',fname_all$dstype[[IDSTYPE_1x1]]$mod[[imod]]$var[[ivar]]$expt[[IEXPT_HISTORICAL]]$period[[IPERIOD_TRAINING_PERIOD]]$stdcal)

			bno = basename( fname_base_orig, '.nc' )
			dno = dirname ( fname_base_orig )

			bnho = basename( fname_base_hist_orig, '.nc' )
			dnho = dirname ( fname_base_hist_orig )
			}

		for( iseas in 1:nseas ) {

			return_plots[[iexpt]][[iperiod]][[iseas]] = list()

			if( ! is_last_mod ) {
				#---------------------------------------------
				# Model predicted change after bias correction
				#---------------------------------------------
				fname_hist = paste0( '../', '/', dnh, '/', bnh, '.', seas[iseas], '.all.nc' )
				fname_fut  = paste0( '../', '/', dn,  '/', bn,  '.', seas[iseas], '.all.nc' )

				print(paste(fname_hist, fname_fut))

				if( filesize( fname_hist ) < 10 ) 
					stop(paste("Error, did not find fname hist after bc:", fname_hist ))
				if( filesize( fname_fut ) < 10 ) 
					stop(paste("Error, did not find fname fut after bc:", fname_fut ))

				mppng_source_file( fname_fut )

				print('-------------------------- final model-predicted change:')
				rv = get_delta( fname_fut, fname_hist, modd, ivar )
				print('-------------------------- ')

				print('---')

				#--------------------------------
				# Original model predicted-change
				#--------------------------------
				fname_hist_orig = paste0( dnho, '/', bnho, '.', seas[iseas], '.all.nc' )
				fname_fut_orig  = paste0( dno,  '/', bno,  '.', seas[iseas], '.all.nc' )

				if( filesize( fname_hist_orig ) < 10 ) 
					stop(paste("Error, did not find fname hist_orig before bc:", fname_hist_orig ))
				if( filesize( fname_fut_orig ) < 10 ) 
					stop(paste("Error, did not find fname fut_orig before bc:", fname_fut_orig ))

				print('-------------------------- orig native grid model-predicted change:')
				rvo = get_delta( fname_fut_orig, fname_hist_orig, modd, ivar, regrid=TRUE )
				print('-------------------------- ')
				}
			else
				{
				#--------------------------------------------------------------
				# Set up rv and rvo from ret_all. rvo is panel 1, rv is panel 2
				# Panel 3 is handled below
				# ret_all[[imod]][[ivar]][[iexpt]][[iperiod]][[iseason]][[3]]$x|y|z
				# In panel 1, all models are on the xvals_com, yvals_com grid
				#--------------------------------------------------------------
				rvo = list()
				rvo$xvals = xvals_com
				rvo$yvals = yvals_com
				mmea_delta = array(0., dim=c(length(xvals_com), length(yvals_com)))
				nmod2add = length( ret_all )
				for( ixmod in 1:nmod2add ) {
					mmea_delta = mmea_delta + ret_all[[ixmod]][[ivar]][[iexpt]][[iperiod]][[iseas]][[1]]$z
					}
				rvo$delta = mmea_delta / nmod2add

				rv = list()
				#------------------------------------------
				# In panel 2, all mods are on the same grid
				#------------------------------------------
				rv$xvals = ret_all[[1]][[ivar]][[iexpt]][[iperiod]][[iseas]][[2]]$x
				rv$yvals = ret_all[[1]][[ivar]][[iexpt]][[iperiod]][[iseas]][[2]]$y
				mmea_delta = array(0., dim=c(length(rv$xvals), length(rv$yvals)))
				for( ixmod in 1:nmod2add ) {
					print(paste("loc b, shape mmea_delta:", 
						pv(dim(mmea_delta)), 
						'shape addand:',
						pv(dim(ret_all[[ixmod]][[ivar]][[iexpt]][[iperiod]][[iseas]][[2]]$z))))
					mmea_delta = mmea_delta + ret_all[[ixmod]][[ivar]][[iexpt]][[iperiod]][[iseas]][[2]]$z
					}
				rv$delta = mmea_delta / nmod2add
				}

			zrange = range( c( rv$delta, rvo$delta ), na.rm=TRUE )

			xlim = range( rv$xvals )
			ylim = range( rv$yvals )

			if( ivar == IVARTYPE_PR ) {
				dv = make_discretevals( 'bluegreenorangebrownREV', seq(-65,65,by=10 ))
				dv$values2label = c( seq(-65, -5, by=20 ), seq(5, 65, by=20))
				}
			else if( ivar == IVARTYPE_DTR ) {
				dv = make_discretevals( 'pretty', seq(-4,4,by=0.5 ))
				dv$values2label = seq(-4, 4, by=1.0 )
				}
			else
				{
				dv = make_discretevals( 'prism_temp', seq(0,10,by=0.25 ))
				dv$values2label = seq(0, 10, by=1)
				}

			#-------------------------------------------
			# Plot orig model-predicted change before BC
			#-------------------------------------------
			cfh = 1.6
			cfyo = 0.6
			if( test_no_plots ) 
				mplot( c(0,1), c(0,1), main=paste('Orig change,', seas[iseas]))
			else
				ccontour( rvo$xvals, rvo$yvals, rvo$delta, 
					discretevals=dv,
					xlim=xlim, ylim=ylim,
					zrange=zrange,
					contour_show=FALSE,
					cbar_fudge_yoffset=cfyo,
					cbar_fudge_height=cfh,
					raster=TRUE,
					main=paste('Orig change,', seas[iseas]))
			return_plots[[iexpt]][[iperiod]][[iseas]][[1]] = list( x=rvo$xvals, y=rvo$yvals, z=rvo$delta )
			col_map = grey(0.2)
			map('state', add=TRUE, col=col_map )
			map('world', add=TRUE, col=col_map )
			mppng_next_frame()

			#-------------------------------------
			# Plot model-predicted change after BC
			#-------------------------------------
			if( test_no_plots ) 
				mplot( c(0,1), c(0,1), main=paste('Post ds change,', seas[iseas]))
			else
				ccontour( rv$xvals, rv$yvals, rv$delta, 
					xlim=xlim, ylim=ylim,
					zrange=zrange,
					discretevals=dv,
					cbar_fudge_yoffset=cfyo,
					cbar_fudge_height=cfh,
					contour_show=FALSE,
					raster=TRUE,
					main=paste('Post d/scale change,', seas[iseas]))
			return_plots[[iexpt]][[iperiod]][[iseas]][[2]] = list( x=rv$xvals, y=rv$yvals, z=rv$delta )
			map('state', add=TRUE, col=col_map )
			map('world', add=TRUE, col=col_map )
			mppng_next_frame()

			#------------------------------------------
			# Plot difference in model-predicted change
			#------------------------------------------
			if( ivar == IVARTYPE_PR ) {
				diffratio_pct = FALSE
				just_difference = TRUE	# NOTE: just differences in percentage points
				zrangerat = c(-30, 30 )
				dv = make_discretevals( 'wetdry', seq(-33, 33, by=6),
					left_arrow_col='orange', right_arrow_col='purple' )
				main = 'Difference in change [pctage-pts]'
				}
			else if( ivar == IVARTYPE_DTR ) {
				diffratio_pct = FALSE
				just_difference = TRUE
				dv = make_discretevals( 'bluereddesat', seq(-1.1, 1.1, by=0.2))
				main = 'Difference in change [degC]'
				}
			else
				{
				diffratio_pct = FALSE
				just_difference = TRUE
				zrangerat = c(-3, 3 )
				dv = make_discretevals( 'bluered', seq(-3.25, 3.25, by=0.5))
				dv$values2label = c( seq(-3.25, 0.25, by=1), seq( 0.25, 3.25, by=1))
				main = 'Difference in change [degC]'
				}

			#-------------------------------------------
			# Need to clip orig field to BC field extent
			#-------------------------------------------
			ix0 = wherenearest( rv$xvals[1], rvo$xvals )
			nnx = length( rv$xvals )
			ix1 = wherenearest( rv$xvals[nnx], rvo$xvals )

			trvo = rvo
			trvo$xvals = rvo$xvals[ix0:ix1]
			trvo$delta = rvo$delta[ix0:ix1,]

			if( (iexpt==2) && (iperiod==1) && (iseas==1)) {
				nxc = length(trvo$xvals)
				nyc = length(trvo$yvals)
				not_mv_mask_dat2 = array( TRUE, dim=c(nxc,nyc))
				#---------------------------------------------
				# Make mask showing not-mv vals on coarse grid
				# Go through coarse grid. If center on fine
				# grid is missing, this value on coarse grid
				# is missing.
				#---------------------------------------------
				print('making MV mask...')
				for( jj in 1:nyc )
				for( ii in 1:nxc ) {
					ix = wherenearest( trvo$xvals[ii], rv$xvals )
					iy = wherenearest( trvo$yvals[jj], rv$yvals )
					if( is.na( rv$delta[ix,iy] ) ) 
						not_mv_mask_dat2[ii,jj] = FALSE
					}
				print('done')
				}


			if( test_no_plots ) {
				mplot( c(0,1), c(0,1), main=main )
				rvrat = rv
				}
			else
				{
				rvrat = calc_2d_ratio( rv$xvals, rv$yvals, rv$delta,
					trvo$xvals, trvo$yvals, trvo$delta,
					regrid=TRUE,
					diffratio_pct=diffratio_pct, 
					just_difference=just_difference,
					not_mv_mask_dat2=not_mv_mask_dat2 )
				ccontour( rvrat$xvals, rvrat$yvals, rvrat$ratio,
					xlim=xlim, ylim=ylim,
					discretevals=dv,
					cbar_fudge_yoffset=cfyo,
					contour_show=FALSE,
					cbar_fudge_height=cfh,
					raster=TRUE,
					main=main )
				}
			return_plots[[iexpt]][[iperiod]][[iseas]][[3]] = list( x=rvrat$xvals, y=rvrat$yvals, z=rvrat$ratio )

			map('state', add=TRUE, col=col_map )
			map('world', add=TRUE, col=col_map )
			add_mean( rvrat$ratio )

			if( iseas == 4 ) {
				if( is_last_mod ) 
					modname_loc = 'MMEA'
				else
					modname_loc = mm$modname_cmip5
				stitle = paste(modname_loc, vv$varname_cmip5, period_pat_label[iperiod], EXPT_NAMES[iexpt] )
				title( outer=TRUE, stitle, cex.main=1.7 )
				}

			mppng_next_frame()
			}
		}

	return( return_plots ) # Our returned plots ... [[expt]][[period]][[season]][[3]]$x|y|z
}

#======================================================================================
domodel = function( fname_all, modd, imod, is_last_mod=FALSE, ret_all=NA ) {

	nvars = length( modd$var )

	retval = list()
	for( ivar in 1:nvars ) {
		retval[[ivar]] = domodel_var( fname_all, modd, imod, ivar, is_last_mod=is_last_mod, ret_all=ret_all )
		}

	return( retval )
}

#======================================================================================
calc_dat = function( opts ) {

	dat = list()

	modd = make_mod_descrip( location, LOC_NASA, LOC_CIRRUS )
	nmod = length( modd$model )

	modnames = array(' ', nmod)
	for( imod in 1:nmod )
		modnames[imod] = modd$model[[imod]]$modname_cmip5

	nvars = length(  modd$var )
	varnames = array(' ', nvars)
	for( ivar in 1:nvars ) 
		varnames[ivar] = modd$var[[ivar]]$varname_cmip5

	nexpt = length( modd$expt )
	expt = array(' ', nexpt)
	for( iexpt in 1:nexpt )
		expt[iexpt] = EXPT_NAMES[iexpt]

	period = period_pat
	nperiods = length( period )

	fname_all        = make_all_filenames( modd )

	hierarchy = list( c(modnames,'MMEA'), varnames, expt[2:3], period_pat_label )

	#---------------------------------------------------
	# Set up multiple png plots per page in a html bunch
	#---------------------------------------------------
	ppp_x = 3
	ppp_y = 4
	plots_per_page = ppp_x * ppp_y
	npages_tot = (nmod+1) * nvars * 2 * nperiods
	multi_page_png( myname, plots_per_page, npages_tot,
		width=810, height=900,
		header_html='Comparing original vs final model-predicted change',
		hierarchy=hierarchy )

	par(mfrow=c(ppp_y,ppp_x), mar=c(3,1,2,0), mgp=c(0,0.2,0), oma=c(0,0,3,0) )

	ret_all = list()
	for( imod in 1:nmod ) {	# plus one cuz last is MMEA

		print(paste("========================== WORKING ON MOD", imod, "/", nmod, '+++++++++++++++++++' ))
		bn = basename( myname, ".R" )

		ret_all[[imod]] = domodel( fname_all, modd, imod, is_last_mod=(imod==nmod+1), ret_all=ret_all )
		}

	# At this point we have ret_all[[imod]][[ivar]][[iexpt]][[iperiod]][[iseason]][[3]]$x|y|z

	return(dat)
}

#======================================================================================
plotup = function( dat, opts, po ) {

}

#======================================================================================
doit = function() {

	use_cache = FALSE
	cache_fname = paste(myname,'_Rcache', sep='' )
	cache_file_exists = (filesize(cache_fname) > 10)

	#-----------------
	# Set data options
	#-----------------
	opts = list()

	#--------------------
	# Use cache, if asked
	#--------------------
	if( use_cache ) {
		if( cache_file_exists ) {
			print(paste("loading data from cache file", cache_fname))
			load( file=cache_fname )
			print('...done')
			do_calc_dat = FALSE
			}
		else
			do_calc_dat = TRUE
		}
	else
		do_calc_dat = TRUE

	#-------------------
	# Calculate the data
	#-------------------
	if( do_calc_dat )
		dat = calc_dat( opts )
		
	#-----------------------------------------------------
	# Save to cache if requested and doesn't exist already
	#-----------------------------------------------------
	if( use_cache && (! cache_file_exists))
		save( file=cache_fname, dat, opts )
			
	#---------------------------------
	# Set plot options and plot result
	#---------------------------------
	po = list()
	plotup( dat, opts, po )
}

#======================================================================================

doit()
